package asw.asw810.context;

import asw.asw810.client.Client;
import asw.asw810.client.connector.ServiceClientProxy;
import asw.asw810.service.Service;
import asw.util.logging.AswLogger;

import java.net.InetAddress;
import java.util.logging.Logger;

/* Application context per il servizio Service e il client Client. */ 
public class ApplicationContext {
	
	/* logger */ 
	private Logger logger = AswLogger.getInstance().getLogger("asw.asw810.context"); 

	/* posizione del server */ 
	// private static String SERVER_ADDRESS = "192.168.56.99"; 
	private static String SERVER_ADDRESS = "localhost"; 

	/* porta del server */ 
	private static int SERVER_PORT = 6789; 

	/* il singleton per l'application context */ 
	private static ApplicationContext instance = null;  
    
	/* Costruttore dell'application context (privato per singleton). */ 
	private ApplicationContext() {
		logger.info("Creazione dell'Application Context"); 
	}
	
    public static synchronized ApplicationContext getInstance() { 
        if (instance==null) { 
        	instance = new ApplicationContext(); 
        }
        return instance; 
    }

    /* Factory method per il servizio Service. 
     * Restituisce un remote proxy al servizio. */ 
    public Service getService() {
		logger.info("ApplicationContext: getService()");
    	Service proxy = null;
    	try {
    		InetAddress address = InetAddress.getByName(SERVER_ADDRESS); 
    		int port = SERVER_PORT; 
    		proxy = new ServiceClientProxy(address, port);
    	} catch(Exception e) {
    		e.printStackTrace();
    	}
    	return proxy; 
    }
    
    /* Factory method per il client Client.  
     * Ogni volta viene restituito un nuovo client. */ 
    public Client getClient() {
		logger.info("ApplicationContext: getClient()");
		Client client = new Client();
    	client.setService( this.getService() );
        return client; 
    }

}

