package asw.asw810.server.connector;

import asw.asw810.service.Service;
import asw.util.logging.AswLogger;

import java.net.*; 
import java.util.logging.Logger;

/* remote proxy lato server per il servizio */ 
public class ServiceServerProxy {
    private Service service;             // il vero servizio 
    private int port;                    // porta per il servizio 

	/* logger */ 
	private Logger logger = AswLogger.getInstance().getLogger("asw.asw810.server.connector"); 

	public ServiceServerProxy(Service service, int port) {
        this.service = service;    this.port = port; 
    }

    public void run() { 
		logger.info("Remote Proxy: Server started");
        try {
            /* crea la socket su cui ricevere le richieste */ 
            DatagramSocket socket = new DatagramSocket(this.port); 
            byte[] buffer = new byte[1000]; 
            while (true) {
                /* aspetta un datagramma di richiesta */ 
                DatagramPacket requestPacket = 
                    new DatagramPacket(buffer, buffer.length); 
                socket.receive(requestPacket);    // bloccante 
                /* estrai la richiesta dal datagramma di richiesta */ 
                String request = new String( requestPacket.getData(), 
                		                     requestPacket.getOffset(), 
                		                     requestPacket.getLength() ); 
        		logger.info("Server Proxy: received request: " + request);
                /* la richiesta ha la forma "operazione#parametro" */
                /* estrae operazione e parametro */
                String op = this.getOp(request);
                String arg = this.getParam(request); 
                /* chiedi lerogazione del servizio ed ottieni la risposta */ 
                String reply = this.executeOperation(op, arg); 
        		logger.info("Server Proxy: sending reply: " + reply);

                /* crea il datagramma di risposta */ 
                /* la risposta ha la forma "risposta" */
                byte[] replyMessage = reply.getBytes(); 
                DatagramPacket replyPacket = 
                    new DatagramPacket( replyMessage, replyMessage.length, 
                                                          requestPacket.getAddress(), 
                                                          requestPacket.getPort() );
                /* invia il datagramma di risposta */ 
                socket.send(replyPacket);    // non bloccante 
            }
        } catch(Exception e) {
        	e.printStackTrace(); 
        }
    }
    
    /* estrae l'operazione dalla richiesta */ 
    private String getOp(String request) {
        /* la richiesta ha la forma "operazione#parametro" */
        int sep = request.indexOf("#"); 
        String op = request.substring(0,sep);
        return op; 
    }
    
    /* estrae il parametro dalla richiesta */ 
    private String getParam(String request) {
        /* la richiesta ha la forma "operazione#parametro" */
        int sep = request.indexOf("#"); 
        String arg = request.substring(sep+1); 
        return arg; 
    }
    
    /* gestisce la richiesta del servizio corretto al servente */ 
    private String executeOperation(String op, String arg) { 
        String reply = null; 

        if ( op.equals("alpha") ) {
    		logger.info("Server Proxy: calling alpha(" + arg + ")"); 
            reply = this.service.alpha(arg);
    		logger.info("Server Proxy: alpha(" + arg + ") ==> " + reply);
        }

        return reply; 
    }

}

